########################################################################
# Fichier menuPackage.tcl 
# 
# 12/05/97 (diam@ensta.fr) : Rajout MenuPopup de J.L COMMEAU
# 09/07/96 (diam@ensta.fr) :  -tearoff 0 au leu de 1 dans la proc Menu
# 12/06/96 (diam@ensta.fr) : 
#   proc MenuCommand { menuName label command  args} 
#   avec args de la forme : -state disable -<option> <valeur> ...
# 11/04/96 (diam@ensta.fr) : procedure MenuCascadeEditGlob
# 
#  partir du chapitre Menu du livre de Brent WELCH
# 
# Todo :
# amliorer MenuBind pour afficher automatiquement la chaine accelerator
# du type "Esc-Ctr-{" lorseque la squence est "<Escape-Control-braceright>"


package provide menu 0.1

proc MenuSetup { {frame .menubar} } {
    global Menu
    # report -v -c frame
    frame $frame
#      pack $frame -side top -fill x
    set Menu(menubar) $frame
    set Menu(uid) 0
    
    return $Menu(menubar)
}

proc Menu { label {side left}} {
    global Menu
    if [info exists Menu(menu,$label)] {
        error "Menu $label already defined"
    }
    # Create the menubutton and its menu
    set name $Menu(menubar).mb$Menu(uid)
    set menuName $name.menu
    incr Menu(uid)
    set mb [menubutton $name -text $label -menu $menuName]
    pack $mb -side $side
    set menu [menu $menuName -tearoff 0]
    
    # Remember the widget name under a variable derived from the label.
    # This allows MenuBind to be passed the label instead of the widget.
    set Menu(menu,$label) $menu
    return $menu
}

proc MenuCommand { menuName label command args} {
    global Menu
    if [catch {set Menu(menu,$menuName)} menu] {
        error "No such menu: $menuName"
    }
    $menu add command -label $label -command $command 
    
    if [llength $args] {
        eval {$menu entryconfigure $label} $args
    }
}


proc MenuCheck { menuName label var { command  {} } } {
    global Menu
    if [catch {set Menu(menu,$menuName)} menu] {
        error "No such menu: $menuName"
    }
    $menu add check -label $label -command $command \
        -variable $var
}

proc MenuRadio { menuName label var {value {}} { command  {} } } {
    global Menu
    if [catch {set Menu(menu,$menuName)} menu] {
        error "No such menu: $menuName"
    }
    if {[string length $value] == 0} {
        set value $label
    }
    $menu add radio -label $label -command $command \
        -value $value -variable $var
}

proc MenuSeparator { menuName } {
    global Menu
    if [catch {set Menu(menu,$menuName)} menu] {
        error "No such menu: $menuName"
    }
    $menu add separator
}

proc MenuCascade { menuName label } {
    global Menu
    if [catch {set Menu(menu,$menuName)} menu] {
        error "No such menu: $menuName"
    }
    if [info exists Menu(menu,$label)] {
        error "Menu $label already defined"
    }
    set sub $menu.sub$Menu(uid)
    incr Menu(uid)
    menu $sub -tearoff 0
    $menu add cascade -label $label -menu $sub
    set Menu(menu,$label) $sub
}

proc MenuBind { what sequence menuName label } {
    global Menu
    if [catch {set Menu(menu,$menuName)} menu] {
        error "No such menu: $menuName"
    }
    if [catch {$menu index $label} index] {
        error "$label not in menu $menuName"
    }
    set command [$menu entrycget $index -command]
    bind $what $sequence $command
    $menu entryconfigure $index -accelerator $sequence
}
proc MenuEntryConfigure { menuName label args } {
    global Menu
    if [catch {set Menu(menu,$menuName)} menu] {
        error "No such menu: $menuName"
    }
    eval {$menu entryconfigure $label} $args
}

proc MenuCascadeEditGlob {menuName pattern} {
    global env
    if ![info exist env(EDITOR)] {set env(EDITOR) stead}
    
    set firstTime 1
    set files [lsort [glob -nocomplain $pattern] ]
    foreach f $files  {
    
        if {$firstTime} {
            set label "$env(EDITOR) [file dirname $f]/..."
            set label "$env(EDITOR) $pattern..."
            MenuCascade  $menuName $label
            set firstTime 0
        }
        if [file isdirectory $f] continue
        MenuCommand  $label  [file tail $f] "exec $env(EDITOR) $f &"  
    }
  
}

# MenuPopup --
# 
# S'utilise comme Menu, mais le packing n'est pas fait.
# 
proc MenuPopup { label } {
global Menu
  if [info exists Menu(menu,$label)] {
    error "Menu $label already defined."
  }
  set Menu(menu,$label) [menu .menupopup$label -tearoff 0]
  return $Menu(menu,$label)
}

proc MenuSampler {} {
    MenuSetup
    Menu Sampler1
    MenuCommand Sampler1 Hello! {puts "Hello, World!"}
    MenuCheck Sampler1 Boolean foo {puts "foo = $foo"}
    MenuSeparator Sampler1
    MenuCascade Sampler1 Fruit
    MenuCascadeEditGlob Sampler1 ~/.X*
    MenuRadio Fruit apple fruit
    MenuRadio Fruit orange fruit
    MenuRadio Fruit kimi fruit
    Menu Sampler2
    MenuCommand Sampler2 Hello! {puts "Hello, World!"}
    MenuCheck Sampler2 Boolean foo {puts "foo = $foo"}
    MenuSeparator Sampler2
    MenuCascade Sampler2 Colors
    MenuCascadeEditGlob Sampler2 ~/.*rc
    MenuRadio Colors blue Colors
    MenuRadio Colors white Colors
    MenuRadio Colors red Colors
}

