#!/bin/sh
# the next line restarts using wish \
exec tclsh7.6 "$0" ${1+"$@"}


proc main {} {
global test

  Pist_SetupMicroAppli test
  
  report -t debut
  source [file join $test(setup) ..  parsarg.tcl]
  
  test_2
  
  report -t fin

}

########################################################################
proc test_1 {} {
    
    set args {-flag1 -opt2 val2 -opt3 val3 -- argum1 argum2}
    
    report "\n# Liste  analyser"
    report -v args

    report "\n# Variables d'entre"
    report -v arga VAR_OPT3
    
    set cmd {Parse_ArgSpec  args arga {
       flag1 flag
       
       opt2  "defaultVal2"
       
       o3   -
       opt3 {
              -variable VAR_OPT3
              -default "DEF3"
       }
    }}
    report "\n# Commande d'analyse :\n$cmd"
    eval $cmd
    report "\n# Liste analyse"
    report -v args 

    report "\n# Variables de sortie"
    report -v arga VAR_OPT3

}
########################################################################
proc test_2 {} {
    # Test de switch non implment :
    
    set args {-flag2 argum1 argum2}
    
    report "\n# Liste  analyser"
    report -v args

    report "\n# Variables d'entre"
    report -v arga  VAR_OPT3
    
    set cmd {Parse_ArgSpec  args arga {
       flag1 flag
       
       opt2  "defaultVal2"
       
       o3   -
       opt3 {
              -variable VAR_OPT3
              -default "DEF3"
       }
    }}
    report "\n# Commande d'analyse :\n$cmd"
    if [catch {eval $cmd} msg] {
       report "\n# Message d'erreur (normal) : \n$msg"
    }
    report "\n# Liste analyse"
    report -v args 

    report "\n# Variables de sortie"
    report -v arga VAR_OPT3

}

########################################################################
proc test_3 {} {
    # Test de switch non implment :
    
    set args {-flag1 -op2 val2  argum1 argum2}
    
    report "\n# Liste  analyser"
    report -v args

    report "\n# Variables d'entre"
    report -v arga  VAR_OPT3
    
    set cmd {Parse_ArgSpec2  args arga {
       flag1 flag
       flag2 flag
       
       opt2  "defaultVal2"
       opt3   defaultVal3
       opt4   {{}}
    }}
    report "\n# Commande d'analyse :\n$cmd"
    if [catch {eval $cmd} msg] {
       report "\n# Message d'erreur (normal) : \n$msg"
    }
    report "\n# Liste analyse"
    report -v args 

    report "\n# Variables de sortie"
    report -v arga VAR_OPT3

}


# Pist_SetupMicroAppli -- installe la librairie pist 
# 
# argument : 
# 
#  - appli : nom du tableau a initialiser (i.e gred, stead, ...)
# 
# 
# sortie : 
# 
#  positionnement des variables suivantes :
#   - appli(exe) : nom absolu de cet excutable (en vue de le re-sourcer)
#   - appli(pistsetup) et env(PIST_SETUP) : installation de PIST
#   - appli(setup) et env(PIST_SETUP) (NON) : installation de PIST
#   - auto_path : insertion de appli(setup) et appli(pistsetup)/lib
# 
# description :
# 
#  Le fonction principale de cette procedure est de le repertoire 
#  d'installation de la librairie PIST utilise.
#  Cette application est du type mono fichier, et PIST est la seule 
#  librairie utilisee
# 
#  Version simplifie de "miniappli:setup" :
#  - ne suivant pas les liens,
#  - ne gre pas les options.
# 
# 
proc Pist_SetupMicroAppli appliName {

global env auto_path argc argv tcl_platform
upvar #0 $appliName appli
   
   cd [file dirname [info script]]
   set appli(setup) [pwd]
   
   # On mmorise le nom absolu de cette application
   set appli(exe) [info script]
   if {[file pathtype $appli(exe)] == "relative"} {
      set appli(exe) [file join [pwd] $appli(exe)]
   }
   
   # Recherche de la librairie PIST
   if {[info exists env(PIST_SETUP)]} {
       set appli(pistsetup) $env(PIST_SETUP)
   } else {
       
       set dirToScan [list   \
          $appli(setup) \
          [file dir $appli(setup)] \
          [file dir [file dir $appli(setup)]] \
          [file dir [file dir [file dir $appli(setup)]]] \
          [file dir [file dir [file dir [file dir $appli(setup)]]]] \
       ]
       foreach dir $dirToScan {
          
          set pistDirs [glob -nocomplain [file join $dir pist]* ]
          if {[llength $pistDirs] == 0} {
              continue
          } 
          
          foreach name [lsort -decreasing  $pistDirs] {
             if [file isdir [file join $name lib]] {
                set pistSetup $name
                break
             }
          }

          if [info exists pistSetup] break
        }
       
        if ![info exists pistSetup] {
          error "Librairie PIST introuvable  partir de $appli(pistsetup) "
          exit
        }
       
        set appli(pistsetup) $pistSetup
        # set env(PIST_SETUP) $appli(pistsetup)
   }
   set auto_path [linsert $auto_path 0  \
             $appli(pistsetup)  [file join $appli(pistsetup) lib]]
   package require pist
   Pist_PackageRequireAll
   
   parray appli

}



main

