# $Id: mailto.tcl,v 1.1.1.1 1997/03/28 14:49:03 diam Exp $
#
#	Copyright (c) 1996 Australian National University and
#	Sun Microsystems
#
#--------
#
# Handlers for mailto: URLs.  Included are a basic handler which uses
# SMTP directly, and a hypertool handler which uses exmh.

package provide mailto [lindex {$Revision: 1.1.1.1 $} 1]

proc mailto_init {} {
    global surfit

    # Define a hypertool handler using exmh, if configured

    if {[info exists surfit(CFGmailto_handler)] && [string match *exmh* $surfit(CFGmailto_handler)]} {
	PRregister_scheme mailto Exmh_mailto PRparser_generic 80
	Hypertool_register SMTP {} exmh exmh {}
    } else {
	# Generic-brand version
	catch {
	    PRregister_scheme mailto SMTP_protocol PRparser_generic 20
	}
    }
}

proc Exmh_mailto {state method data put_data} {
    upvar #0 $state var
    upvar 2 $data data_return

    if {![catch {set win [SurfIt_hyperpage $var(cookie)]}]} {
	upvar #0 PS$win altState
    }

    # Normal URL parsing does not apply to mail addresses
    regsub {mailto:} $var(url) {} addr

    # There doesn't appear to be a means of supplying the msg form
    # directly from a program.  So, write the form into the cache
    # file

    Cache_add_document PRfd $state	;# Kludge to make cache open a file
    Cache_append_data $state "To: $addr
cc:
Subject:
"
    if {[info exists altState(referer)]} {
	Cache_append_data $state "X-URL: $altState(referer)\n"
    }
    Cache_append_data $state "--------\n"
    if {$method == "put"} {
	Cache_append_data $state $data
    }
    Cache_close $state

    Hypertool_send SMTP "Msg_Compose -form $var(cachefile)"

    # Finished with the file, so remove it
    # This is safe since exmh has already read the file contents
    Cache_expire_document $var(url)

    return {}
}


##########################################################################
## Authors: Jonathan Perret and Gerald Lester				##
## Purpose: This sends SMTP mail	
##    Date: December 27, 1995						##
##########################################################################

## Send SMTP to mail host.
proc SMTP_sendmail {smtphost toList from subject body {trace 0}} {
	if $trace then {
		puts stdout "Connecting to $smtphost:25"
	}
	set sockid [socket $smtphost 25]
	puts $sockid "HELO $smtphost\nMAIL From:<$from>"
	flush $sockid
	set result [gets $sockid]
	if $trace then {
		puts stdout "MAIL From:<$from>\n\t$result"
	}
	foreach to $toList {
	    puts $sockid "RCPT To:<$to>"
	    flush $sockid
	}
	set result [gets $sockid]
	if $trace then {
		puts stdout "RCPT To:<$to>\n\t$result"
	}
	puts  $sockid "DATA "
	flush $sockid
	set result [gets  $sockid]
	if $trace then {
		puts stdout "DATA \n\t$result"
	}
	puts  $sockid "From: <$from>"
	puts  $sockid "To: <$to>"
	puts  $sockid "Subject: $subject"
	puts  $sockid "\n"
	foreach line [split $body  "\n"] {
		puts  $sockid "[join $line]"
	}
	puts  $sockid "."
	puts  $sockid "QUIT"
	flush $sockid
	set result [gets  $sockid]
	if $trace then {
		puts stdout "QUIT\n\t$result"
	}
	close $sockid 
	return;
}

###
### Protocol handler for mailto: URLs
### Author: Steve Ball
###

# Hopefully this protocol handler will be overridden by a 
# mail user agent acting as a hypertool.

# The mailto: URL causes a mail message to be composed and sent
# to the given address.  This protocol never returns data to be
# rendered

catch {
    PRregister_scheme mailto SMTP_protocol PRparser_generic 20
}

proc SMTP_protocol {state method data put_data} {
    upvar #0 $state var
    upvar 2 $data data_return

    # Normal URL parsing does not apply to mail addresses
    regexp {mailto:([^@]+)@(.*)} $var(url) all user host

    # Construct the composition window

    set p [toplevel .mailto$state]
    wm title $p "Send Mail to User $user at $host"

    label $p.subjLabel -text "Subject: "
    entry $p.subject -width 40
    grid $p.subjLabel -row 0 -column 0
    grid $p.subject -sticky ew -row 0 -column 1

    label $p.toLabel -text "To: "
    entry $p.to -width 40
    grid $p.toLabel -row 1 -column 0
    grid $p.to -sticky ew -row 1 -column 1
    $p.to insert 0 "$user@$host"

    label $p.ccLabel -text "Cc: "
    entry $p.cc -width 40
    grid $p.ccLabel -row 2 -column 0
    grid $p.cc -sticky ew -row 2 -column 1

    label $p.bodyLabel -text "Message to send:"
    text $p.body
    grid $p.bodyLabel -row 3 -column 0 -columnspan 2 -sticky ew
    grid $p.body -row 4 -column 0 -columnspan 2 -sticky news

    set q [frame $p.ctl]
    grid $q -row 5 -column 0 -columnspan 2 -sticky w
    button $q.send -text "Send Message" -command "SMTP_send $p $user $host"
    button $q.cancel -text "Cancel Message" -command "SMTP_cancel $p $user $host"
    grid $q.send $q.cancel -sticky w

    return {}
}

proc SMTP_send {win user host} {
    global surfit

    if {[info exists surfit(CFGsmtphost)]} {set host $surfit(CFGsmtphost)}

    SMTP_sendmail $host  [$win.to get] $surfit(CFGusername)  [$win.subject get]  [$win.body get 1.0 end]

    destroy $win
}

proc SMTP_cancel {win user host} {
    if {![tk_dialog $win.confirm {Cancel Message?}  "Are you sure that you want to cancel sending this message  to $user at $host?" {} 1 Yes No]} {
	destroy $win
    }
}



