########################### EPSF PACKAGE ######################################
###############################################################################
# Format de la variable "epsf" :
# epsf(c)		: Nom du canvas
# epsf(toplevel)	: Nom de la fenetre de setup de haut niveau 
# epsf(file)		: nom du fichier postscript genere 
# epsf(rotate)		: Orientattion de la page
# epsf(mode)		: Type du mode couleur 
# epsf(zone_imprimee)	: Attribut de la zone imprimee
# epsf(ok)		: Indicateur de validation de la configuration
# epsf(width)		: Largeur de la zone imprimee
# epsf(height)		: Hauteur de la zone imprimee
# epsf(geometry)	: Taille et position de la fenetre contenant 
#  			  le canvas
# epsf(pref,xxx)	: Preferences
# epsf()		: 
################################################################################


# Procedure de configuration de l'impression postscript
proc Epsf_Dialog { ca } {
  global epsf

  set epsf(c) $ca

  # definition de la fenetre setup    
  if [catch {set epsf(toplevel) [toplevel .top -width 100 -height 100]}] {
    raise $epsf(toplevel)
    return
  } else {
    set top $epsf(toplevel)
    wm title $top "Epsf Dialog Box"
    wm geometry $top +100+100 
  }
 
  # option file
  set fi [ frame $top.file ]
  if { [info exist epsf(file)] == 0 } {
    Epsf_Default file
  }

  label $fi.l -text "File Name :"
  entry $fi.e -textvariable epsf(file)
  button $fi.bu -text "Browse" -command {
   tk_dialog .msg "Message" "Browser not available ..." "" 0 "OK"
  }
  pack $fi -side top
  pack $fi.l $fi.e $fi.bu -side left


  # option rotate  
  set or [ frame $top.rotate ]
  if { [info exist epsf(rotate)] == 0 } {
    Epsf_Default rotate
  }
  
  label $or.l -text "Orientation :"
  radiobutton $or.portrait -text "Portait" -variable epsf(rotate) \
  -value false
  radiobutton $or.landscape -text "Landscape" -variable epsf(rotate) \
  -value true
  pack $or -side top -fill x
  pack $or.l -anchor nw
  pack $or.portrait $or.landscape -side left


  # option colormode
  set cm [ frame $top.mode ]
  if { [info exist epsf(mode)] == 0 } {
    Epsf_Default colormode
  }
  
  label $cm.l -text "Color Mode :"
  radiobutton $cm.color -text "Color" -variable epsf(mode) -value color
  radiobutton $cm.gray -text "Gray" -variable epsf(mode)  -value gray
  radiobutton $cm.mono -text "Mono" -variable epsf(mode) -value mono
  pack $cm -side top -fill x
  pack $cm.l -anchor nw
  pack $cm.color $cm.gray $cm.mono -side left


  # option width et height
  set wi_he [ frame $top.wi_he ]
  if { [info exist epsf(zone_imprimee)] == 0 } {
    Epsf_Default zone_imprimee
  }
  
  label $top.wi_he.l -text "Printed Zone : "
  menubutton $wi_he.mb -textvariable epsf(zone_imprimee) -menu $wi_he.mb.menu -relief raise -width 8 
  menu $wi_he.mb.menu -tearoff 0
  $wi_he.mb.menu add radio -label All -variable epsf(zone_imprimee) \
    -value All
  $wi_he.mb.menu add radio -label Window -variable epsf(zone_imprimee) \
    -value Window
  pack $wi_he -side top -fill x
  pack $wi_he.l $wi_he.mb -side left


  # Definition des boutons de validation
  set va [ frame $top.valid ]
  set epsf(ok) 0
  button $va.ok -text "OK" -command { set epsf(ok) 1}
  button $va.defaut -text "Default" -command { Epsf_Default all } 
  button $va.cancel -text "Cancel" -command { set epsf(ok) 0 } 
  pack $va -side bottom -fill x
  pack $va.ok $va.defaut $va.cancel -side right

  # Attente de la validation
  tkwait variable epsf(ok)
  
  # prise en cours de la zone d'impression apres avoir quitter la fenetre
  switch -- $epsf(zone_imprimee) {
    All { 
      set epsf(width) [ $epsf(c) cget -width ]
      set epsf(height) [ $epsf(c) cget -height ]
    }
    Window {
      set geom [wm geometry [winfo parent $epsf(c) ] ]
      regsub -all {[x\+-]} $geom " " epsf(geometry)
      set epsf(width) [lrange $epsf(geometry) 0 0]
      set epsf(height) [lrange $epsf(geometry) 1 1]
    }
  }

  switch -- $epsf(ok) {
    1 {
      if { [File:ExistAndReplace $epsf(file)] } {
        destroy $epsf(toplevel)
      } else {
      
        # generation de l'impression
        $epsf(c) postscript \
          -file $epsf(file) \
          -rotate $epsf(rotate) \
          -colormode $epsf(mode) \
          -width $epsf(width) \
          -height $epsf(height) \
          -pagewidth 210.m \
          -pageheight 297.m
#           -colormap \
#           -fontmap \
#           -pageanchor \
#           -pageheight \
#           -pagewidth \
#           -pagex \
#           -pagey \

        destroy $epsf(toplevel)
        tk_dialog .msg "Message" "Printing done in file : $epsf(file) ..."\
          "" 0 "OK"
      }
    }
    0 {
      destroy $epsf(toplevel)
    }
  }

}


# Procedure d'initialisation des preferences
proc Epsf_Init { } {
  global epsf
  
set epsf(pref,file) "mon_fich_pref.eps"
set epsf(pref,mode) color
set epsf(pref,zone_imprimee) All

}


# Procedure d'initialisation des valeurs par defaut
proc Epsf_Default { quoi } {
  global epsf
  
  switch -- $quoi {
    colormap {
    }
    colormode {
      if { [info exist epsf(pref,mode)] } {
        set epsf(mode) $epsf(pref,mode)
      } else {
        set epsf(mode) color
      }
    } 
    file {
      if { [info exist epsf(pref,file)] } {
        set epsf(file) $epsf(pref,file)
      } else {
        set epsf(file) "default.eps"
      }
    }
    fontmap {
    }
    zone_imprimee {
      if { [info exist epsf(pref,zone_imprimee)] } {
        set epsf(zone_imprimee) $epsf(pref,zone_imprimee)
      } else {
        set epsf(zone_imprimee) All
      }
    }
    pageanchor {
    }
    pageheight {
    }
    pagewidth {
    }
    pagex {
    }
    pagey {
    }
    rotate {
      if { [info exist epsf(pref,rotate)] } {
        set epsf(rotate) $epsf(pref,rotate)
      } else {
        set epsf(rotate) false
      }
    }
    x {
    }  # x
    y {
    }  # y
    all {
    
      if { [info exist epsf(pref,mode)] } {
        set epsf(mode) $epsf(pref,mode)
      } else {
        set epsf(mode) color
      }
      
      if { [info exist epsf(pref,file)] } {
        set epsf(file) $epsf(pref,file)
      } else {
        set epsf(file) "default.ps"
      }
      
      if { [info exist epsf(pref,rotate)] } {
        set epsf(rotate) $epsf(pref,rotate)
      } else {
        set epsf(rotate) false
      }

      if { [info exist epsf(pref,zone_imprimee)] } {
        set epsf(zone_imprimee) $epsf(pref,zone_imprimee)
      } else {
        set epsf(zone_imprimee) All
      }
      
    }	# all
  }
  
}
