# $Id: html.tcl,v 1.1.1.1 1997/03/28 14:49:03 diam Exp $
#
#	Copyright (c) 1996 Australian National University and
#	Sun Microsystems.
#	See http://surfit.anu.edu.au/copyright.html for a full
#	description of terms and conditions.
#
#	Copyright (c) 1996 Jeffrey Hobbs
#
#####
#
# HTML Megawidget
#
# Displays HTML documents.  Elements are included to scroll documents,
# control history mechanism, etc.
#
# Namespace Usage:
#
# This module reserves the prefixes "html" and "HM" for use with procedures 
# and variables.  Megawidgets also use variables prefixed by the name of 
# the widget pathname.

# Based upon html_library.tcl.  Notice from that package:

## Simple HTML display library by Stephen Uhler (stephen.uhler@sun.com)
## Copyright (c) 1995 by Sun Microsystems
## Version 0.3 Fri Sep  1 10:47:17 PDT 1995
##
## See the file "license.terms" for information on usage and redistribution
## of this file, and for a DISCLAIMER OF ALL WARRANTIES.

# Mega-widget template from Jefrey Hobbs.

package provide html [lindex {$Revision: 1.1.1.1 $} 1]

#####

# html --
#
# html megawidget class creation command.
#
# Arguments:
#	path	widget path to create
#	args	configuration options
#
# Return Value:
#	Widget pathname.
#
# Methods supported:
#
#	delete		-- delete elements
#	element		-- element definitions
#	insert		-- insert elements
#	load		-- load HTML text
#	reset		-- clear the widget
#	stylesheet	-- stylesheet definitions
#	xview		-- x scroll view
#	yview		-- y scroll view
#
# State Variables
#
# The state variable for the megawidget has the same name as the
# widget pathname.
# Each nested Text widget, including the main Text, have a state
# variable named HM$path.
#
# To Do:
#	Progressive loading of data
#	Index scheme

proc html {w args} {
  global HMstyle_default HMuse_document

  frame $w -class Html

  upvar \#0 $w data
  # Create widget state.
  # history is for the document history stack
  # mru is for a list of Most Recently Used documents
  array set data {
    history {}
    mode html
    mru {}
    cmdprefix	HMtag_
    progresscommand HMnoop
    linkvisitedcommand HMlinkVisited
    linksetupcommand HMlinkSetup
    updatecommand HMupdate
    updInterval 10
  }
  if $HMuse_document {
    set data(linkcommand) {document loaduri}
    set data(linkvisitedcommand) DOC_isCached
    set data(imagecommand) {document loaduri -command {HMloadImage %s}}
  }
  set data(main) .$w.t
  set data(mainwin) $w.t
  set data(banner) $w.banner
  set data(status) $w.status

  # Create status/feedback windows
  frame $w.status
  label $w.status.t
  frame $w.status.scale -width 104 -height 18 -borderwidth 2 -relief ridge
  frame $w.status.scale.value -background red -width 0 -height 14
  grid $w.status.scale.value -sticky w
  grid $w.status.scale -row 0 -column 1 -sticky e
  grid propagate $w.status.scale 0
  grid columnconfig $w.status.scale 0 -weight 1
  grid $w.status.t -row 0 -column 0 -sticky w
  grid columnconfig $w.status 0 -weight 1

  # Create hyperwindow
  set text [text $w.t -yscrollcommand "$w.sy set" -xscrollcommand "$w.sx set" -wrap word]
  rename $w .$w
  rename $text $data(main)
  if {[string comp {} $args] && \
      [catch {eval HMwidget_configure $w $args} err]} {
    destroy $w
    unset data
    return -code error $err
  }
  ## Gotta watch what you put in this list
  bindtags $text [list $w $text Widget Text [winfo toplevel $w] all]
  interp alias {} $w {} HMwidgetEval $w $data(main)
  interp alias {} $text {} HMwidgetEval $w $data(main)

  HMinit_win $w $data(main) $text
  HMwidget_stylesheet $w $data(main) add browser $HMstyle_default
  bind $w <Destroy> [list HMwidgetDestroy $w]

  $w.status.t conf -text {}

  scrollbar $w.sx -command "$text xview" -orient h -bd 1 -highlightt 1
  scrollbar $w.sy -command "$text yview" -orient v -bd 1 -highlightt 1

  grid $text $w.sy -sticky news
  grid $w.sx -sticky ew
  grid $w.status - -sticky ew
  grid columnconfig $w 0 -weight 1
  grid rowconfig $w 0 -weight 1

  return $w
}

proc HMwidgetEval { w text args } {
  upvar \#0 $w data

  switch -glob -- [lindex $args 0] {
    {} {
      return -code error "wrong \# args: should be \"$w option ?arg ...?\""
    }
    conf*	{ eval HMwidget_configure $w [lrange $args 1 end] }
    cg*		{ eval HMwidget_cget $w [lrange $args 1 end] }
    load	{ eval HMwidget_load $w $text [lrange $args 1 end] }
    reset	{ eval HMwidget_reset $w $text [lrange $args 1 end] }
    stylesheet	{ eval HMwidget_stylesheet $w $text [lrange $args 1 end] }
    default	{ eval $text $args }
  }
}

proc HMwidgetDestroy w {
  global HMtimerScripts
  upvar \#0 $w data

  catch {
    foreach s [array names HMtimerScripts $data(main)*] {
      unset HMtimerScripts($s)
    }
  }

  catch { upvar #0 ${w}_ssheet ssheet; unset ssheet }
  catch { upvar #0 $data(main)_fontCache font; unset font }
  catch { rename $w {}; unset data }
}

proc HMwidget_configure { w args } {
  upvar \#0 $w data

  if [string comp {} $args] {
    set truth {^(1|yes|true|on)$}
    for {set i 0;set num [llength $args];set cargs {}} {$i<$num} {incr i} {
      set arg [string tolower [lindex $args $i]]
      set val [lindex $args [incr i]]
      switch -glob -- $arg {
	-imagecommand	{ set data(imagecommand) $val }
	-linkcommand	{ set data(linkcommand) $val }
	-linksetupcommand	{ set data(linksetupcommand) $val }
	-linkvisitedcommand	{ set data(linkvisitedcommand) $val }
	-mode	{ set data(mode) $val }
	-progresscommand	{ set data(progresscommand) $val }
	-tagcommand	{ set data(cmdprefix) $val }
	-title	{ set data(title) $val }
	-updatecommand	{ set data(updatecommand) $val }
	-updateinterval	{ set data(updInterval) $val }
	default	{ lappend cargs $arg $val }
      }
    }
    if {[string comp {} $cargs] && \
	[catch {eval $data(main) config $cargs} err]} {
      return -code error $err
    }
    return
  } else {
    set conf [$data(main) config]
    lappend conf \
	[list -imagecommand	{} {} {}	$data(imagecommand)]
	[list -linkcommand	{} {} {}	$data(linkcommand)]
	[list -linksetupcommand	{} {} {}	$data(linksetupcommand)]
	[list -linkvisitedcommand	{} {} {}	$data(linkvisitedcommand)]
	[list -mode	{} {} {}	$data(mode)]
	[list -progresscommand	{} {} {}	$data(progresscommand)]
	[list -tagcommand	{} {} {}	$data(cmdprefix)]
	[list -title	{} {} {}	$data(title)]
	[list -updatecommand	{} {} {}	$data(updatecommand)]
	[list -updateinterval	{} {} {}	$data(updInterval)]
    return [lsort $conf]
  }
}

proc HMwidget_cget { w args } {
  if {[llength $args] != 1} {
    return -code error "wrong \# args: should be \"$w cget option\""
  }
  upvar \#0 $w data
  switch -glob -- $args {
    -imagecommand	{ return $data(imagecommand) }
    -linkcommand	{ return $data(linkcommand) }
    -linksetupcommand	{ return $data(linksetupcommand) }
    -linkvisitedcommand	{ return $data(linkvisitedcommand) }
    -mode	{ return $data(mode) }
    -progresscommand	{ return $data(progresscommand) }
    -tagcommand	{ return $data(cmdprefix) }
    -title	{ return $data(title) }
    -updatecommand	{ return $data(updatecommand) }
    -updateinterval	{ return $data(updInterval) }
    default	{ eval $data(main) cget $args }
  }
}

# HMwidget_reset --
#
# Delete all data from the widget and reset the state, ready to
# load a new document.

proc HMwidget_reset { w t args } {
    upvar #0 $w data
    upvar #0 HM$data(main) var

    $t delete 1.0 end
    set tags [$t tag names]
    # Preserve font and colour tags
    regsub -all {(font[^ ]*)} $tags {} tags
    regsub -all {(colour[^ ]*)} $tags {} tags
    eval $t tag delete $tags
    eval $t mark unset [$t mark names]

    foreach v [array names data dest:*] {
	unset data($v)
    }

    HMinit_win $w $data(main) $data(mainwin)
    HMprogress $w reset {}
    HMprogress $w update 0
}

# HMwidget_load --
#
# load method for html megawidget.  This method can accept
# either raw HTML text or pre-parsed Tcl commands, as indicated
# by the submethod.  The option -end indicates whether any
# more HTML is to follow.
#
# Arguments:
#	w	widget state
#	args	data
#
# Valid Submethods:
#	begin	commence loading a document.  Arg is base.
#	data	display a segment of the HTML document
#	postdata display text which has already been parsed
#	progress display progress report on document status
#	error	handle an error occuring during document load
#	end	document has finished.

proc HMwidget_load { w t args } {
    upvar #0 $w data

    switch [lindex $args 0] {
	begin {
	    switch [llength $args] {
		2 {
		    $w configure -mode [HMmode_map [lindex $args 1]]
		}
		3 {
		    $w configure -mode [HMmode_map [lindex $args 1]]
		}
		default {error {too many arguments}}
	    }
	    HMwidget_reset $w $t
	    if {[string compare {} [lindex $args 2]]} {
		set data(base) [lindex $args 2]
	    }
	    return
	}
	data {
	    if {[llength $args] != 2} {
		error {wrong # args}
	    }
	    switch $data(mode) {
		html {
		    if ![string compare {} [lindex $args 1]] return;	# nothing to do
		    set html [html_parse [lindex $args 1] -start $data(start) -end no -unusedvariable ${w}(unused) -headcommand $data(headcommand) -bodycommand $data(bodycommand) -updatecommand $data(updatecommand) -updateinterval $data(updInterval)]
		    set data(start) 0
		}
		text {
		    $t insert end [lindex $args 1]
		    return
		}
		image {
		    return
		}
		none {
		    return
		}
	    }
	}
	parsed {
	    if {[llength $args] != 2} {
		error {wrong # args}
	    }
	    error {not implemented}
	}
	end {
	    if {[llength $args] > 1} {
		error {wrong # args}
	    }
	    switch $data(mode) {
		html {
		    if {!$data(start) && ![info exists data(unused)]} {
			# html_parse doesn't handle empty input very well
			set html "HMnoop \{"
		    }
		    append html [html_parse {} -start $data(start) -end yes -unusedvariable ${w}(unused) -headcommand $data(headcommand) -bodycommand $data(bodycommand) -updatecommand $data(updatecommand) -updateinterval $data(updInterval)]
		    HMprogress $w update $data(total_tags)
		}
		image {
		    return
		}
		text -
		none return
	    }
	}
	progress {
	    eval HMprogress $w [lrange $args 1 end]
	    return
	}
	error {
	    eval HMerror $w [lrange $args 1 end]
	    return
	}
	default {error "unknown submethod \"[lindex $args 0]\""}
    }
    set data(stop) 0
    if {[catch {incr data(total_tags) [expr int([llength $html] / 6)]}]} {
	puts stderr "\nError in parsing HTML.  Bad parsed HTML is:\n"
	puts stderr $html
	puts stderr "\n\n"
	return
    }
    regsub -all {\\state\\} $html $w html
    # We may still be evaluating a previous chunk of HTML.
    # Add this lot to the list of HTML being evaluated.
    set nextChunk [incr data(chunkCntr)]
    set data(chunk$nextChunk) $html
    append data(chunk$nextChunk) "\nHMnextEval $w $nextChunk"
    if !$data(currEval) {
	set data(currEval) 1
	eval $data(chunk$nextChunk)
    }
}

# HMnextEval --
#
# This procedure is called when a chunk of HTML has finished evaluating.
# We can clean up the data and evaluate the next lot of HTML, if any.

proc HMnextEval {state chunk} {
    upvar #0 $state data

    unset data(chunk$chunk)
    incr chunk
    if [info exists data(chunk$chunk)] {
	puts stderr [list HMnextEval $state [expr $chunk - 1] scheduling chunk $chunk]
	eval $data(chunk$chunk)
    } else {
	puts stderr [list HMnextEval $state [expr $chunk - 1] no further chunks]
	set data(currEval) 0
    }
}

# HMmode_map --
#
# Map MIME content-types to megawidget modes.

proc HMmode_map type {
    switch -glob $type {
	text/html {
	    return html
	}
	text/* {
	    return text
	}
	image/* {
	    return image
	}
	default {
	    return none
	}
    }
}

###
### Automatic connection to document package
###

set HMuse_document 0

if {[lsearch [package names] document] >= 0} {
    set HMuse_document 1

    # DOC_target instructs the document package to use the target
    # as the command for handling loads

    document type handler text/html DOC_target -quality 10
}

array set HMtype2image {
    image/jpeg	photo
    image/gif	photo
    image/x-portable-pixmap	photo
    image/x-xbitmap	bitmap
}

proc HMloadImage {win event args} {
    global HMtype2image
    puts stderr [list HMloadImage $win $event $args]

    switch $event {
	begin {
	    upvar #0 $win var
	    set var(imagetype) $HMtype2image([lindex $args 0])
	    set var(file) [DOC_getFile [lindex $args 1]]
	}
	end {
	    upvar #0 $win var
	    set img [image create $var(imagetype) ${win}_img]
	    $img configure -file $var(file)
	    $win configure -image $img
	}
	data {
	    # Update progress meter
	}
	error {
	    # Update progress meter
	    $win configure -foreground red
	}
	progress {
	    # Update progress meter
	}
	default {error "unknown submethod \"$event\""}
    }
}

# HMwidget_stylesheet --
#
# Manipulate stylesheets for the widget
#
# Arguments:
#	w	widget state
#	t	text widget
#	args	data
#
# Return Value:

proc HMwidget_stylesheet {w t args} {
    upvar #0 $w data

    switch [lindex $args 0] {
	add {
	    if {[llength $args] > 3} {
		error "too many arguments"
	    }
	    set level [lindex $args 1]
	    set ssheet [html_parse_css1 [lindex $args 2] -command html_apply_style]
	    regsub -all {\\state\\} $ssheet "$t ${w}_ssheet" ssheet
	    eval $ssheet
	    set data(ssheet) ${w}_ssheet
	}
	info {
	}
	remove {
	}
	default {
	    error "unknown stylesheet method \"[lindex $args 0]\""
	}
    }
}

# HMRenderHead --
#
# Handles HTML elements that appear in the document HEAD.
# These elements do not cause text to be rendered on the screen.
# A separate procedure allows lightweight handling of these
# elements.
#
# Arguments:
#	state	Megawidget state variable
#	tag	HTML tag to be processed
#	not	Whether this tag opens or closes the element
#	param	attributes
#	text	Text following tag until next tag

proc HMRenderHead {state tag not param text} {
    #puts stderr [list HMRenderHead $state $tag $not $param $text]
    upvar #0 $state data

    catch {$data(cmdprefix)$not$tag $state attr text} err
    #puts stderr [list head tag processing returned $err]
    if [string compare {} $text] {
	# Synthesize paragraph
	HMRenderBody $state p {} {} $text
    }
}

# HMRenderBody --
#
# Renders a HTML element that appears in the BODY 
# in the display.
#
# State Variables:
#
# $state (data) contains the parser state.
# HM$data(current) contains the state for the current window
# (HM prefix is used since $state may be the same as window name).
# $data(ssheet) contains the current stylesheet
#
# Arguments:
#	state	Megawidget state variable
#	tag	HTML tag to be processed
#	not	Whether this tag opens or closes the element
#	param	attributes
#	text	Text following tag until next tag

proc HMRenderBody {state tag not param text} {
    global HMfont_symbol_map
    upvar #0 $state data
    #puts stderr [list HMRenderBody $state $tag $not $param $text, current tags $data(tags)]
    if $data(stop) return
    upvar #0 HM$data(current) var
    upvar #0 $data(ssheet) ssheet

    incr data(tagID)

    # Close previous block when starting new block
    # Exception is nesting blocks
    catch {
	if [regexp nesting $ssheet($tag,display)] {
	    switch $not {
		{} {
		    lappend data(nesting) $data(curr-block)
		}
		/ {
		    # Close current block within nested element if different to nested block
		    if {[string compare $tag $data(curr-block)] && [string compare {} $data(curr-block)]} {
			# Implicitly close currently open block element
			HMRenderBody $state $data(curr-block) / {} {}
		    }
		    set data(curr-block) [lindex $data(nesting) end]
		    set data(nesting) [lreplace $data(nesting) end end]
		}
	    }
	} elseif {[string compare / $not] && [regexp {block|list-item} $ssheet($tag,display)] && [string compare {} $data(curr-block)]} {
	    # Implicitly close currently open block element
	    HMRenderBody $state $data(curr-block) / {} {}
	}
    }

    # Handle closing blocks before popping tag stacks
    catch {switch -regexp -- $ssheet($tag,display),$not {
	{^block[^,]*,/$} -
	{^list-item,/$} {
	    set data(curr-block) {}
	    if ![regexp {.*\.0$} [$data(current) index $var(S_insert)]] {
		$data(current) insert $var(S_insert) \n
	    }
	    set bottom margin0
	    catch {set bottom [subst $ssheet($tag,margin-bottom)]}
	    $data(current) insert $var(S_insert) \n $bottom
	    set data(trim_ws) 1

	    # Pop margin settings
	    set var(margin-left) [lreplace $var(margin-left) end end]
	    set var(margin-right) [lreplace $var(margin-right) end end]
	    set data(tags) [lreplace [lreplace $data(tags) end end] end end]
	}
    }}

    # Common attribute handling
    if [HMextract_param $param id] {
	puts stderr [list got ID $id]
    }
    if [HMextract_param $param class] {
	puts stderr [list got class $class]
    }

    # Push or pop property stacks
    # In essence, this replaces HMstack and inlines the code to save a proc call
    catch {
	switch $not {
	    / {
		# Pop
		foreach prop $ssheet($tag) {
		    set data($prop) [lreplace $data($prop) end end]
		    set data(tags) [lreplace $data(tags) end end]
		}
		if $ssheet($tag,font-change) {
		    set data(font) [lreplace $data(font) end end]
		    set data(tags) [lreplace $data(tags) end end]
		}
	    }
	    {} {
		# Push
		foreach prop $ssheet($tag) {
		    lappend data($prop) [subst $ssheet($tag,$prop)]
		    lappend data(tags) [lindex $data($prop) end]
		}
		if $ssheet($tag,font-change) {
		    lappend data(font) [HMset_font $state]
		    lappend data(tags) [lindex $data(font) end]
		}
	    }
	}
    }

    # special tag processing
    catch {$data(cmdprefix)$not$tag $state $param text} err
    #puts stderr [list special tag processing returned $err]

    # Handle block elements after pushing tag stacks
    catch {switch -regexp -- $ssheet($tag,display),$not {
	{^block[^,]*,$} -
	^list-item,$ {
	    if ![regexp nesting $ssheet($tag,display)] {
		set data(curr-block) $tag
	    }
	    set top margin0
	    catch {set top [subst $ssheet($tag,margin-top)]}
	    if [regexp {.*\.0$} [$data(current) index $var(S_insert)]] {
		set prevline [$data(current) index "$var(S_insert) - 1 char"]
		$data(current) tag add $top $prevline
	    } else {
		$data(current) insert $var(S_insert) \n
		$data(current) insert $var(S_insert) \n $top
	    }
	    set data(trim_ws) 1

	    # Set left, right margins
	    set left [lindex $var(margin-left) end]
	    catch {set left [subst $ssheet($tag,margin-left)]}
	    lappend var(margin-left) $left
	    lappend data(tags) $left
	    set right [lindex $var(margin-right) end]
	    catch {set right [subst $ssheet($tag,margin-right)]}
	    lappend var(margin-right) $right
	    lappend data(tags) $right

	    # Optimsation: unroll switch to avoid redundant if check
	    if ![string compare list-item $ssheet($tag,display)] {
		# Insert bullet or number
		switch -regexp -- [lindex $data(list-style) end] {
		    {url\([^)]\)} {
			#puts stderr {ignoring url()}
		    }
		    disc -
		    circle -
		    square {
			# Need to be able to allow this to be configured to other fonts
			set bullet $HMfont_symbol_map([lindex $data(list-style) end])
			set bullet_tag symbol
		    }
		    decimal -
		    lower-roman -
		    upper-roman -
		    lower-alpha -
		    upper-alpha {
			set point [lindex $data(list-counter) end]
			set data(list-counter) [lreplace $data(list-counter) end end [incr point]]
			set bullet [HMcvtListCounter $point [lindex [lindex $data(list-style) end] 0]]
			set bullet_tag {}
		    }
		    default {
			# default to disc
			set bullet $HMfont_symbol_map(disc)
			set bullet_tag symbol
		    }
		}
		$data(current) tag raise symbol
		if [regexp inside [lindex $data(list-style) end]] {
		    # Add inside bullet
		    $data(current) insert $var(S_insert) "$bullet " "$bullet_tag $data(tags)"
		} else {
		    # Assuming current tag pushed left-margin
		    set prevleft [lindex $var(margin-left) [expr [llength $var(margin-left)] - 2]]
		    $data(current) tag configure tab$data(tagID) -lmargin1 [$data(current) tag cget $prevleft -lmargin1] -tabs [$data(current) tag cget $left -lmargin1]
		    $data(current) insert $var(S_insert) $bullet "$bullet_tag tab$data(tagID) $data(tags)" \t tab$data(tagID) 
		}
	    }
	}
	inline {
	    set data(trim_ws) 0
	}
    }}

    # Text transformations
    catch {set text [string [lindex $data(text-transform) end] $text]}

    if {[lindex $data(white-space) end] != "nowrap"} {
	# Collapse white space
	if $data(trim_ws) {
	    set text [string trimleft $text]
	    set data(trim_ws) 0
	}
	regsub -all "\[ \t\r\n\]+" $text " " text
    }

    # Insert text
    #puts stderr [list adding text $text with tags $data(tags)]
    $data(current) insert $var(S_insert) $text $data(tags)
}

# HMfontRelative --
#
# Returns a tag name of a tag configured with a new font relative to 
# the current font.
#
# Arguments:
#	selector	selector for base font
#	args		option/value pairs

proc HMfontRelative {state args} {
    global HMfontCache
    upvar #0 $state data
    upvar #0 $data(current)_fontCache cache

    # Determine new font
    foreach prop [array names data font-*] {
	set font($prop) [lindex $data($prop) end]
    }
    foreach {opt value} $args {
	switch -- $opt {
	    -relsize {
		set font(font-size) [expr int($font(font-size) * $value / 100)]
	    }
	    -size {
		set font(font-size) $value
	    }
	    -weight {
		set font(font-weight) $value
	    }
	    -family {
		set font(font-family) $value
	    }
	    -style {
		set font(font-style) $value
	    }
	}
    }
    set relfont -*-$font(font-family)-$font(font-weight)-$font(font-style)-*-*-$font(font-size)-*-*-*-*-*-*-*

    if [catch {set cache($relfont)} tag] {
	if [info exists HMfontCache($relfont)] {
	    $data(current) tag configure font$data(tagID) -font $relfont
	} else {
	    HMprogress $state fontLoading "Loading font $relfont"
	    if [catch {$data(current) tag configure font$data(tagID) -font $relfont}] {
		HMprogress $state fontError "Unable to load font $relfont"
	    } else {
		HMprogress $state fontLoaded "Loaded font $relfont"
		set HMfontCache($relfont) {}
	    }
	}
	set cache($relfont) font$data(tagID)
    } else {
	return $tag
    }
}

# HMset_font --
#
# Return a tag name where the tag is configured with the font given by 
# the font- properties in the given state.

proc HMset_font {state} {
    global HMfontCache
    #puts stderr [list HMset_font $state]
    upvar #0 $state data
    upvar #0 $data(current)_fontCache cache

    set font -*-[lindex $data(font-family) end]-[lindex $data(font-weight) end]-[lindex $data(font-style) end]-*-*-[lindex $data(font-size) end]-*-*-*-*-*-*-*
    if [catch {set cache($font)} tag] {
	if [info exists HMfontCache($font)] {
	    $data(current) tag configure font$data(tagID) -font $font
	} else {
	    HMprogress $state fontLoading "Loading font $font"
	    if [catch {$data(current) tag configure font$data(tagID) -font $font}] {
		HMprogress $state fontError "Unable to load font $font"
	    } else {
		HMprogress $state fontLoaded "Loaded font $font"
		set HMfontCache($font) {}
	    }
	}
	set cache($font) font$data(tagID)
    } else {
	return $tag
    }
}

# HMset_margin --
#
# Returns a tag which sets the margin from the side of the window.
# Margins are not inherited, but they are, in effect, relative to the
# containing block's margin.
#
# Arguments:
#	state	global state array for HTML parser
#	side	left or right
#	current	tag name of current margin
#	opt	how margin is to be changed
#	value	amount of change

proc HMset_margin {state side current opt value} {
    #puts stderr [list HMset_margin $state $side $current $opt $value]
    upvar #0 $state data
    upvar #0 HM$data(current) var

    if {$side == "left"} {
	set indent [$data(current) tag cget $current -lmargin1]
	set tagopts {-lmargin1 %s -lmargin2 %s}
    } else {
	set indent [$data(current) tag cget $current -rmargin]
	set tagopts {-rmargin %s}
    }

    switch -- $opt {
	-indent {
	    incr indent $value
	}
	-relindent {
	    # Relative to current font
	    incr indent [expr int([lindex $data(font-size) end] * $value / 100)]
	}
    }

    regsub -all {%s} $tagopts $indent tagopts
    eval $data(current) tag configure $side$data(tagID) $tagopts
    return $side$data(tagID)
}

# HMcvtListCounter --
#
# Converts the integer value to the given list style, such as lower-roman.
# Defaults to decimal.

proc HMcvtListRep {letter rep} {
    set ret {}
    while {$rep} {
	append ret $letter
	incr rep -1
    }
    return $ret
}

proc HMcvtListCounter {value style} {
    switch -- $style {
	lower-roman {
	    # This will only work up to 3999.
	    # Can't put bars over letters anyway.
	    # This should be rewritten!
	    if {$value >= 1000} {
		return [HMcvtListRep m [expr $value / 1000]][HMcvtListCounter [expr $value % 1000] $style]
	    }
	    switch -regexp [expr $value / 100] {
		9 {
		    return cm[HMcvtListCounter [expr $value % 100] $style]
		}
		{[5-8]} {
		    return d[HMcvtListRep c [expr ($value / 100) - 5]][HMcvtListCounter [expr $value % 100] $style]
		}
		4 {
		    return cd[HMcvtListCounter [expr $value % 100] $style]
		}
		{[1-3]} {
		    return [HMcvtListRep c [expr $value / 100]][HMcvtListCounter [expr $value % 100] $style]
		}
		0 {
		    switch -regexp [expr $value / 10] {
			9 {
			    return xc[HMcvtListCounter [expr $value % 10] $style]
			}
			{[5-8]} {
			    return l[HMcvtListRep x [expr ($value / 10) - 5]][HMcvtListCounter [expr $value % 10] $style]
			}
			4 {
			    return xl[HMcvtListCounter [expr $value % 10] $style]
			}
			{[1-3]} {
			    return [HMcvtListRep x [expr $value / 10]][HMcvtListCounter [expr $value % 10] $style]
			}
			0 {
			    switch -regexp $value {
				9 {return ix}
				{[5-8]} {return v[HMcvtListRep i [expr $value - 5]]}
				4 {return iv}
				default {return [HMcvtListRep i $value]}
			    }
			}
		    }
		}
	    }
	}
	upper-roman {
	    return [string toupper [HMcvtListCounter $value lower-roman]]
	}
	lower-alpha {
	    if {$value > 26} {
		# Return "aa", "ab", etc
		return [HMcvtListCounter [expr int(($value - 1) / 26)] $style][HMcvtListCounter [expr (($value - 1) % 26) + 1] $style]
	    } else {
		return [format %c [expr 96 + $value]]
	    }
	}
	upper-alpha {
	    return [string toupper [HMcvtListCounter $value lower-alpha]]
	}
	default {return $value}
    }
}

proc HMinit_win {state cmd win} {
    global HMtimerScripts HMfont_symbol_map
    upvar #0 $state data
    upvar #0 HM$data(main) var

    HMinit_state $state $cmd $win

    bind $win <Configure> [list HMconfigure_win $state $cmd %W %w]
    catch {wm title [winfo toplevel $win] $data(title)}
    $cmd mark set $var(S_insert) 1.0

    # Create simple tags

    # This may need to be platform-specific
    $cmd tag configure margin0 -font -misc-nil-*-*-*-*-0-*-*-*-*-*-*-*
    $cmd tag configure hr -font -misc-nil-*-*-*-*-6-*-*-*-*-*-*-* -relief sunken -borderwidth 2 -wrap none -tabs [expr [winfo width $win] - 10]
    append data(configurecommands) "\n$cmd tag configure hr -tabs \[expr \$width - 10\]"
    $cmd tag lower margin0
    $cmd tag configure left0 -lmargin1 0 -lmargin2 0
    $cmd tag configure right0 -rmargin 0

    # Choose bullets
    if [catch {$cmd tag configure symbol -font -*-zapfdingbats-*-*-*-*-12-*-*-*-*-*-*-*}] {
	if [catch {$cmd tag configure symbol -font -*-symbol-*-*-*-*-18-*-*-*-*-*-*-*}] {
	    $cmd tag configure symbol -font -*-times-*-*-*-*-18-*-*-*-*-*-*-*
	    array set HMfont_symbol_map "disc \267 circle \260 square \267"
	} else {
	    array set HMfont_symbol_map "disc \267 circle \157 square \336"
	}
    } else {
	array set HMfont_symbol_map "disc \154 circle \155 square \156"
    }

    $cmd tag configure center -justify center
    $cmd tag configure right -justify right
    $cmd tag configure nowrap -wrap none
    $cmd tag configure overstrike -overstrike true
    $cmd tag configure underline -underline true
    $cmd tag configure flat -relief flat
    $cmd tag configure raised -relief raised
    $cmd tag configure sunken -relief raised
    $cmd tag configure groove -relief raised
    $cmd tag configure ridge -relief raised

    # The dreaded blinking tag
    $cmd tag configure blink -foreground [$cmd cget -foreground]
    set HMtimerScripts($cmd) "
	if \$flipflop {
	    $cmd tag raise blink
	    $cmd tag configure blink -foreground \[$cmd cget -foreground\]
	} else {
	    $cmd tag raise blink
	    $cmd tag configure blink -foreground \[$cmd cget -background\]
	}
    "
    return {}
}

# HMinit_state --
#
# (Re)set the state variable for a widget

proc HMinit_state {state cmd win} {
    puts stderr [list HMinit_state $state $cmd $win]
    upvar #0 $state data
    upvar #0 HM$cmd var

    array set data "current $cmd currentwin $win"
    array set data {
	start 1 currEval 0 chunkCntr 0 updCntr 0 total_tags 1
	white-space 1 tagID 0 trim_ws 0 list-counter 0
	anchor_target {} base {}
	headcommand HMRenderHead bodycommand HMRenderBody 
	configurecommands {}
	curr-block {} nested-block {}
	font-family times font-style r font-weight medium font-size 14
    }
    set data(font) [HMset_font $state]
    set data(tags) $data(font)

    catch {unset var}
    array set var {S_insert Insert margin-left left0 margin-right right0}
}

# HMconfigure_win --
#
# This procedure is called whenever the window changes size

proc HMconfigure_win {state cmd win width} {
    upvar #0 $state var
    if {$var(configurecommands) != {}} {
	eval $var(configurecommands)
    }
}

# HMtimer --
#
# This routine performs periodic functions, such as blink text.

if {![info exists HMtimerScripts]} {
    array set HMtimerScripts {}
}
if {![info exists HMtimerInterval]} {
    set HMtimerInterval 500
}

proc HMtimer {flipflop} {
    global HMtimerScripts HMtimerInterval HMtimerID

    foreach cmd [array names HMtimerScripts] {
	eval $HMtimerScripts($cmd)
    }
    set HMtimerID [after $HMtimerInterval HMtimer [expr !$flipflop]]
}

proc HMtimerStart {} {
    global HMtimerInterval HMtimerID

    catch {after cancel $HMtimerID}
    set HMtimerID [after $HMtimerInterval HMtimer 1]
}

proc HMtimerStop {} {
    global HMtimerID

    catch {after cancel $HMtimerID}
}

HMtimerStart

# HMprogress --
#
# Set progress messages and invoke application-supplied progress script

proc HMprogress {state status {message {}}} {
    upvar #0 $state data

    switch $status {
	update {
	    $data(status).scale.value conf -width [expr int((double($message) / double($data(total_tags))) * 100.0)]
	}
	default {
	    # Should find and use default foreground colour
	    $data(status).t conf -text $message -foreground black
	}
    }

    eval $data(progresscommand) $status [list $message]
}

# HMupdate --
#
# Wrapper for update command.  Also keeps stats for feedback.

proc HMupdate state {
    upvar #0 $state data
    HMprogress $state update [incr data(updCntr) $data(updInterval)]
    update
}

# HMerror --
#
# Display an error message in the status window.

proc HMerror {state errCode {message {}}} {
    upvar #0 $state data

    $data(status).t conf -text $message -foreground red
    # eval $data(errorcommand) $errCode [list $message]
}

#####
#
# Special tag commands
#
#####

proc HMtag_hmstart {state param text} {
    HMprogress $state startRender {}
}

proc HMtag_/hmstart {state param text} {
    HMprogress $state endRender {}
}

proc HMtag_title {state param text} {
    upvar #0 $state data
    upvar $text txt

    wm title [winfo toplevel $data(currentwin)] $txt
    set txt {}
}

proc HMtag_base {state param text} {
    upvar #0 $state data
    if [HMextract_param $param href] {
	set data(base) $href
    }
}

proc HMtag_a {state param text} {
    upvar #0 $state data
    upvar #0 HM$data(current) var
    upvar #0 $data(ssheet) ssheet

    if [HMextract_param $param href] {
	# A source anchor
	set data(anchor_source) $href
	lappend data(tags) [list link:$href]
	catch {eval $data(linksetupcommand) $state $data(current) $data(currentwin) [list $href] [list link:$href]}
	if [eval $data(linkvisitedcommand) [list $href] [list $data(base)]] {
	    # A:visited pseudo-class
	    set data(linkvisited) 1
	    catch {
		foreach prop $ssheet(a:visited) {
		    lappend data($prop) [subst $ssheet(a:visited,$prop)]
		    lappend data(tags) [lindex $data($prop) end]
		}
		if $ssheet(a:visited,font-change) {
		    lappend data(font) [HMset_font $state]
		    lappend data(tags) [lindex $data(font) end]
		}
	    }
	} else {
	    # A:link pseudo-class
	    set data(linkvisited) 0
	    catch {
		foreach prop $ssheet(a:link) {
		    lappend data($prop) [subst $ssheet(a:link,$prop)]
		    lappend data(tags) [lindex $data($prop) end]
		}
		if $ssheet(a:link,font-change) {
		    lappend data(font) [HMset_font $state]
		    lappend data(tags) [lindex $data(font) end]
		}
	    }
	}
    }

    if [HMextract_param $param name] {
	# A destination anchor
	lappend data(anchor_target) $name
	set data(dest:$name) $data(current)
	lappend data(tags) dest:$name
	$data(current) mark set dest:$name "$var(S_insert) - 1 chars"
	$data(current) mark gravity dest:$name left
	if {[info exists data(goto)] && $data(goto) == $name} {
	    unset var(goto)
	    set var(going) $name
	}
    }
}

proc HMtag_/a {state param text} {
    upvar #0 $state data
    upvar #0 $data(ssheet) ssheet

    if [info exists data(anchor_source)] {
	if $data(linkvisited) {
	    set tag a:visited
	} else {
	    set tag a:link
	}
	catch {
	    foreach prop $ssheet($tag) {
		set data($prop) [lreplace $data($prop) end end]
		set data(tags) [lreplace $data(tags) end end]
	    }
	    if $ssheet($tag,font-change) {
		set data(font) [lreplace $data(font) end end]
		set data(tags) [lreplace $data(tags) end end]
	    }
	}
	set data(tags) [lreplace $data(tags) end end]
	unset data(anchor_source)
	unset data(linkvisited)
    }

    # goto this link, then invoke the call-back.

    if {[info exists data(going)]} {
	$data(current) yview dest:$data(going)
	update
	HMwent_to $state $data(current) $data(going)
	unset data(going)
    }

    if [string compare {} $data(anchor_target)] {
	set data(tags) [lreplace $data(tags) end end]
	set data(anchor_target) [lreplace $data(anchor_target) end end]
    }
}

# HMlink_hit --
#
# Handles hyperlink activation.
#
# Arguments:
#	state	widget state variable
#	target	hyperlink target

proc HMlink_hit {state target} {
    upvar #0 $state data

    if [string match #* $target] {
	# Internal destination
	regsub # $target {} target
	HMgoto $state $target
    } else {
	catch {puts stderr "HMlink_hit calling $data(linkcommand) [list $target] [list $data(base)]"}
	catch {eval $data(linkcommand) [list $target] [list $data(base)]}
    }
}

# HMgoto --
#
# The application should call here with the fragment name
# to cause the display to go to this spot.
# If the target exists, go there (and do the callback),
# otherwise schedule the goto to happen when we see the reference.

proc HMgoto {state where {callback HMwent_to}} {
    upvar #0 $state data

    if [info exists data(dest:$where)] {
	$data(dest:$where) see dest:$where
	update
	eval $callback $state $data(dest:$where) [list $where]
	return 1
    } else {
	set data(goto) $where
	return 0
    }
}

# HMwent_to --
#
# We actually got to the spot, so highlight it!
# This should/could be replaced by the application
# We'll flash it orange a couple of times.

proc HMwent_to {state win where {count 0} {color orange}} {
    upvar #0 $state data

    if {$count > 5} return
    catch {$win tag configure dest:$where -foreground $color}
    update
    after 200 [list HMwent_to $state $win $where [incr count] \
				[expr {$color=="orange" ? "" : "orange"}]]
}

# HMlinkSetup --
#
# Application-dependent setup of link tag.
#
# Bugs:
#	Should take configuration from A:active selector in stylesheet

array set HMlinkEvents {
	Enter	{-borderwidth 2 -relief raised }
	Leave	{-borderwidth 2 -relief flat }
	1		{-borderwidth 2 -relief sunken}
	ButtonRelease-1	{-borderwidth 2 -relief raised}
}

proc HMlinkSetup {state cmd win href tagname} {
    global HMlinkEvents
    upvar #0 $state data

    regsub -all % $tagname %% name2
    foreach i [array names HMlinkEvents] {
	eval {$cmd tag bind $tagname <$i>} \{$cmd tag configure \{$name2\} $HMlinkEvents($i)\}
    }
    # Feedback
    $cmd tag bind $tagname <Enter> "+$data(status).t conf -text [list $href] -fg black"
    $cmd tag bind $tagname <Leave> "+$data(status).t conf -text {} -fg black"

    # Hyperlink event
    $cmd tag bind $tagname <ButtonRelease-1> "+HMlink_hit $state [list $href]"
}

# HMlinkVisited --
#
# Returns 1 if the given URL has been recorded before,
# 0 otherwise.
#
# Arguments:
#	href	URL to query

proc HMlinkVisited href {
    global HMlinkArray

    return [info exists HMlinkArray($href)]
}

# HMtag_hr --
#
# Horizontal rules can't be handled by the normal tag processing
# mechanisms.
#
# Current bugs:
#	Doesn't consider right margin

proc HMtag_hr {state param text} {
    upvar #0 $state data
    upvar #0 HM$data(current) var

    # Get current left and right margins
    set left [lindex $var(margin-left) end]
    set right [lindex $var(margin-right) end]

    # Close off previously open block, if any
    if {[string compare {} $data(curr-block)]} {
	# Implicitly close currently open block element
	HMRenderBody $state $data(curr-block) / {} {}
    }

    # End line if non-empty
    if ![regexp {.*\.0$} [$data(current) index $var(S_insert)]] {
	$data(current) insert $var(S_insert) \n
    }

    # Insert horizontal rule
    $data(current) tag raise hr
    $data(current) insert $var(S_insert) \t "hr $data(tags) $left" \n hr
}

# HMtag_img --
#
# Insert image documents into the current document.
#
# Creating widgets can be time-consuming, particularly texts and buttons.
# Caching created widgets is desirable.  However, the text widget destroys
# all embedded widgets when they are deleted.  To workaround this, widgets to be
# embedded are created as peers to the text widget.  A frame is created as a
# child of the text widget.  The widget is then packed in the frame.
# When the text is reset, in readiness for a new document, only the frame is
# destroyed.
#
# For images, buttons are used to display the image since they can also support
# images as anchors.

proc HMtag_img {state param text} {
    puts stderr [list HMtag_img $state $param $text]
    upvar #0 $state data
    upvar #0 HM$data(current) var

    # get alignment
    # NB: should set default alignment from stylesheet
    array set align_map {top top    middle center    bottom bottom}
    set align bottom		;# The spec isn't clear what the default should be
    HMextract_param $param align
    catch {set align $align_map([string tolower $align])}

    # get alternate text
    set alt "<image>"
    HMextract_param $param alt
    set alt [HMmap_esc $alt]

    # get the border width
    # NB: should set default border width from stylesheet
    set border 1
    HMextract_param $param border

    # Create a frame for the image
    set item [frame $data(currentwin).frame$data(tagID) -borderwidth $border]
    $data(current) window create $var(S_insert) -align $align -window $item -pady 2 -padx 2
    # add in all the current tags
    foreach tag $data(tags) {
	$data(current) tag add $tag $item
    }

    # see if we have an image size hint
    # If so, make the frame the "hint" size to put the widget in
    # The stylesheet may also give width & height specification
    if {[HMextract_param $param width] && [HMextract_param $param height]} {
	$item configure -width $width -height $height
	grid propagate $item 0
    }

    # Find or create a button for the image
    # This will be a child of the text widget, but not embedded in the window.
    # It will be destroyed only when the megawidget is destroyed.  
    # There should probably be an expiry mechanism as well to avoid excessive 
    # memory consumption.
    set button $data(currentwin).img$data(tagID)
    catch {button $button}
    $button configure -fg orange -text $alt -image {} -borderwidth 0 -state normal
    grid $button -in $item -sticky news
    raise $button

    # Handle anchor images
    if [info exists data(anchor_source)] {
	bind $button <ButtonRelease-1> "HMlink_hit $state [list $data(anchor_source)]"
    } else {
	$button configure -state disabled -disabledforeground orange
    }

    # set imagemap callbacks
    if [HMextract_param $param ismap] {
	#global HMevents
	# Protect % chars from bind
	regsub -all % $data(anchor_source) %% link
	#foreach i [array names HMevents] {
	#    bind $button <$i> "catch \{%W configure $HMevents($i)\}"
	#}
	bind $button <ButtonRelease-1> "HMlink_hit $state $link?%x,%y"
    } 

    # now callback to the application
    set src {}
    HMextract_param $param src
    # Insert target into imagecommand
    # There should be a better way of passing the target to the handler
    set cmd [format $data(imagecommand) $button]
    catch {eval $cmd [list $src] [list $data(base)]}
    return $button	;# used by the forms package for input_image types
}

#####

# html_parse --
#
# A modified version of the HMparse_html procedure.
# This version accepts incomplete HTML and performs 
# post-processing.
#
# Arguments:
#	html	HTML text
#	args	option/value pairs
#
# Valid Options:
#	-rendercommand	Default HMtest_parse
#	-headcommand	Used instead of -rendercommand. Command to handle HEAD elements
#	-bodycommand	Used with -headcommand. Command to render BODY elements
#	-unusedvariable
#	-starttag	Default hmstart
#	-start		Default true
#	-end		Default true
#	-updatecommand
#	-updateinterval
#	
# Return value:
#	The HTML text translated into Tcl commands.
#
# Current Bugs:
#	Error checking of arguments is not complete.

set HMheadcommands {^(hmstart|html|head|title|link|style|meta|isindex|base|script)$}

proc html_parse {html args} {
    global HMheadcommands

    array set opts {
	-headcommand HMtest_parse 
	-bodycommand HMtest_parse 
	-starttag hmstart
	-updateinterval 10
	-start 1 -end 1
    }
    array set opts $args	;# very rudimentary option processing
    regsub {^(1|yes|true|on)$} $opts(-start) 1 opts(-start)
    regsub {^(0|no|false|off)$} $opts(-start) 0 opts(-start)
    regsub {^(1|yes|true|on)$} $opts(-end) 1 opts(-end)
    regsub {^(0|no|false|off)$} $opts(-end) 0 opts(-end)
    if {[info exists opts(-rendercommand)]} {
	array set opts "-headcommand $opts(-rendercommand) -bodycommand $opts(-rendercommand)"
    }

    regsub -all \{   $html {\&ob;}  html
    regsub -all \}   $html {\&cb;}  html
    regsub -all \\\\ $html {\&bsl;} html
    set w " \t\r\n"	;# white space
    proc HMcl x {return "\[$x\]"}
    set exp <(/?)([HMcl ^$w>]+)[HMcl $w]*([HMcl ^>]*)>
    set sub "\}\n$opts(-bodycommand) {\\2} {\\1} {\\3} \{"
    if {[info exists opts(-unusedvariable)]} {
	upvar #0 $opts(-unusedvariable) unused
	if {[info exists unused]} {
	    # Sanity check
	    if {$opts(-start)} {
		error "specified start of HTML, but found unused HTML text"
	    }
	    regsub -all $exp "$unused$html" $sub html
	    unset unused
	} else {
	    regsub -all $exp $html $sub html
	}
    } else {
	regsub -all $exp $html $sub html
    }

    if {$opts(-start)} {
	set html "$opts(-headcommand) $opts(-starttag) {} {} \{$html\}"
    } else {
	set html [string range $html 2 end]\}
    }

    set cnt 0
    foreach {command tag not param text} $html {
	if {[regexp $HMheadcommands [set tag [string tolower $tag]]]} {set command $opts(-headcommand)}
	append parsed "\n$command \\state\\ [list $tag] [list $not] [list $param] [list [HMmap_esc $text]]"
	if {[info exists opts(-updatecommand)] && [incr cnt] % $opts(-updateinterval) == 0} {
	    append parsed "\n$opts(-updatecommand) \\state\\"
	}
    }
    # Handle non-ending data
    if {[info exists opts(-unusedvariable)] && !$opts(-end)} {
	# Check if the last tag has been split
	if [regexp {([^<]*)(<.*)} $text all t l] {
	    set unused $l
	    set parsed [lreplace $parsed end end $t]
	} else {
	    # Save the last tag for next time 'round
	    set unused [lrange $parsed [expr [llength $parsed] - 5] end]
	    set parsed [lreplace $parsed [expr [llength $parsed] - 5] end]
	    # Rewrite last tag as HTML for re-parsing next time
	    regexp {[^\\]*\\state\\ ([^ ]+) ([^ ]+) ([^ ]+) (.+)} $unused all tag not param text
	    # Remove extraneous braces
	    regexp {^\{([^\}]*)\}$} $not all not
	    regexp {^\{([^\}]*)\}$} $param all param
	    regexp {^\{([^\}]*)\}$} $text all text
	    set unused "<$not$tag $param>$text"
	}
    }
    if {$opts(-end)} {
	append parsed "\n$opts(-headcommand) \\state\\ $opts(-starttag) / {} {}"
    }

    return $parsed
}

# HMnoop --
#
# Consumes arguments, does nothing

proc HMnoop {args} {}

# html_parse_css1 --
#
# Translates a CSS1 stylesheet into Tcl commands
#
# Arguments:
#	ssheet	a style sheet in CSS1 format
#
# Return Value:
#	Tcl commands to implement the style sheet

proc html_parse_css1 {ssheet args} {
    # Process switches
    array set opts {
	-command HMtest_style
    }
    array set opts $args	;# very rudimentary option processing

    # Intercept @import directives
    if {[regexp @import $ssheet]} {
	# Do something about it
	return {}
    }
    # Remove comments
    regsub -all {/\*[^*]*\*/} $ssheet {} ssheet
    regsub -all {([^{]+)({[^}]*})} $ssheet {{\1} \2 } ssheet

    # Process each rule
    set ws " \t\n"
    foreach {selector declaration} $ssheet {
	# Pre-process selectors and declarations
	regsub -all "( |\t|\n)+" [string trim $selector] { } selector
	set selector [string tolower $selector]
	regsub -all "(\[^:$ws\]+)\[$ws\]*:\[$ws\]*(\[^;\]+)(;|$)" $declaration { \1 {\2} } declaration
	set decl {}
	foreach {property value} $declaration {
	    lappend decl $property [string trim $value]
	}

	foreach s [split $selector ,] {
	    append result "\n$opts(-command) \\state\\ [list [string trim $s]] [list $decl]"
	}
    }

    return $result
}

#
# Mappings for CSS1 value names
#

array set HMfont_family_map {
    serif times sans-serif helvetica cursive zapfchancery fantasy zapfdingbats monospace courier
}
array set HMfont_style_map {
    normal {}
    italic i cursive i kursiv i
    oblique o slanted o incline o
}
array set HMfont_abs_size_map {
    xx-small 8 x-small 10 small 12 medium 14 large 18 x-large 24 xx-large 30
}
array set HMfont_rel_size_map {
    smaller 67 larger 150
}
array set HMstyle_ws_map {
    pre nowrap nowrap nowrap
}
array set HMstyle_decoration_map {
    none {} underline underline overline underline line-through overstrike blink blink
}
array set HMstyle_colour_map {
    aqua aquamarine
    black black blue blue
    fuchsia deeppink
    gray grey50 grey grey50 green green 
    lime limegreen
    maroon maroon navy navy olive olivedrab purple purple
    red red
    silver gainsboro
    teal turquoise3
    white white yellow yellow
}
array set HMborder_style_map {
    none none
    solid flat
    dotted flat
    dashed flat
    double ridge
    groove groove
    ridge ridge
    inset sunken
    outset raised
}
array set HMborder_width_map {
    thin 1
    medium 2
    thick 4
}

# html_apply_style --
#
# Apply a style to the hyperwindow state.
#
# BUGS: doesn't do anything about the cascading rules.
#
# Arguments:
#	cmd		the text widget command
#	tag_map		the Text tag map for the hyperwindow
#	selector	the HTML tag this declaration applies to
#	declaration	the CSS1 list of properties

# Generate a script to set up the tags, etc for this style,
# and another script to add the style when rendering.

proc html_apply_style {cmd tag_map selector declarations} {
    global HMfont_family_map HMfont_style_map HMfont_abs_size_map HMfont_rel_size_map HMstyle_ws_map HMstyle_decoration_map
    global HMborder_style_map HMborder_width_map
    upvar #0 $tag_map tmap

    foreach {property value} $declarations {
	switch $property {
	    background {}
	    border {
		#-relief ... -borderwidth ... -bordercolor {}

		if [regexp {([0-9]+[^ ]*)} $value all width] {
		    switch [HMcvtLength $width len] {
			absolute {
			}
			relative {
			    set len 2	;# not supported yet
			}
		    }
		} else {
		    set width medium
		    regexp {(thin|medium|thick)} $value all width
		    set len $HMborder_width_map($width)
		}
		regsub $width $value {} value	;# remove this token from the value

		set style solid
		regexp {(none|dotted|dashed|solid|double|groove|ridge|inset|outset)} $value all style
		regsub $style $value {} value
		set style $HMborder_style_map($style)
		if ![string compare none $style] continue


		# Anything left in the value will be a colour specification
		# Tk can't set the colour of the border
		#set colour [HMcvtColour $cmd [string trim $value]]
		#if {$colour != {}} {
		#    set colour [$cmd tag cget $colour -foreground]
		#    $cmd tag configure border$style$width$colour -borderwidth $len -relief $style -bordercolor $colour
		#    set tagname border$style$width$colour
		#} else {}
		    $cmd tag configure border$style$width -borderwidth $len -relief $style
		    set tagname border$style$width
		set tmap($selector,$property) $tagname
		lappend tmap($selector) $property
	    }
	    border-bottom -
	    border-left -
	    border-right -
	    border-top {
		# Tk can't render sides alone, so map to none
	    }
	    color {
		if {[set ctag [HMcvtColour $cmd $value]] == ""} continue
		set tmap($selector,$property) $ctag
		lappend tmap($selector) $property
	    }
	    clear {}
	    float {}
	    font {}
	    font-family {
		catch {set value $HMfont_family_map($value)}
		set tmap($selector,$property) $value
		lappend tmap($selector) $property
		set tmap($selector,font-change) 1
	    }
	    font-size {
		if [catch {set value $HMfont_abs_size_map($value)}] {
		    if [catch {set value $HMfont_rel_size_map($value)}] {
			switch -- [HMcvtLength $value len] {
			    absolute {
				set tmap($selector,$property) $len
			    }
			    relative {
				set tmap($selector,$property) "\[HMfontRelative $state -relsize $len\]"
			    }
			    default continue
			}
		    } else {
			set tmap($selector,$property) "\[HMfontRelative $state -relsize $value\]"
		    }
		} else {
		    set tmap($selector,$property) $value
		}
		lappend tmap($selector) $property
		set tmap($selector,font-change) 1
	    }
	    font-style {
		catch {set value $HMfont_style_map($value)}
		set tmap($selector,$property) $value
		lappend tmap($selector) $property
		set tmap($selector,font-change) 1
	    }
	    font-variant {
		if ![string compare small-caps $value] {
		    # Emulate small-caps
		    set tmap($selector,text-transform) toupper
		    set tmap($selector,font-size) {[HMfontRelative $state -relsize 80]}
		    lappend tmap($selector) text-transform font-size
		    set tmap($selector,font-change) 1
		}
	    }
	    font-weight {
		# Tk can't give a listing of available fonts, so can't do much here
		# Only have a choice of normal (medium) or bold
		switch -regexp -- $value {
		    [1-9]00 {
			if {$value <= 500} {
			    # Normal
			    set tmap($selector,$property) medium
			} else {
			    # Bold
			    set tmap($selector,$property) bold
			}
		    }
		    lighter -
		    normal {
			set tmap($selector,$property) medium
		    }
		    bolder -
		    bold {
			set tmap($selector,$property) bold
		    }
		    default continue
		}
		lappend tmap($selector) font-weight
		set tmap($selector,font-change) 1
	    }
	    height {}
	    letter-spacing {}
	    line-height {
		#-spacing2 ...
	    }
	    margin {
		switch [llength $value] {
		    1 {}
		    2 {}
		    3 {}
		    4 {}
		    default {}
		}
	    }
	    margin-bottom {
		if {[string index $value 0] == "-"} continue;
		set type [HMcvtLength $value len]
		if {$len == 0} continue;
		switch -- $type {
		    absolute {
			$cmd tag configure margin$len -font -*-courier-*-*-*-*-$len-*-*-*-*-*-*-*
			set tmap($selector,$property) margin$len
		    }
		    relative {
			# This is subst'd at runtime
			set tmap($selector,$property) "\[HMfontRelative \$state -relsize $len\]"
		    }
		    default continue
		}
	    }
	    margin-left {
		if {[string index $value 0] == "-"} continue;
		set type [HMcvtLength $value len]
		if {$len == 0} continue;
		switch -- $type {
		    absolute {
			set tmap($selector,$property) "\[HMset_margin \$state left \$left -indent $len\]"
		    }
		    relative {
			set tmap($selector,$property) "\[HMset_margin \$state left \$left -relindent $len\]"
		    }
		    default continue
		}
		#lappend tmap($selector) $property
	    }
	    margin-right {
		if {[string index $value 0] == "-"} continue;
		set type [HMcvtLength $value len]
		if {$len == 0} continue;
		switch -- $type {
		    absolute {
			set tmap($selector,$property) "\[HMset_margin \$state right \$right -indent $len\]"
		    }
		    relative {
			set tmap($selector,$property) "\[HMset_margin \$state right \$right -relindent $len\]"
		    }
		    default continue
		}
		#lappend tmap($selector) $property
	    }
	    margin-top {
		if {[string index $value 0] == "-"} continue;
		set type [HMcvtLength $value len]
		if {$len == 0} continue;
		switch -- $type {
		    absolute {
			$cmd tag configure margin$len -font -*-courier-*-*-*-*-$len-*-*-*-*-*-*-*
			set tmap($selector,$property) margin$len
		    }
		    relative {
			# This is subst'd at runtime
			set tmap($selector,$property) "\[HMfontRelative \$state -relsize $len\]"
		    }
		}
	    }
	    padding {}
	    padding-bottom {
		#-spacing3 ...
	    }
	    padding-left {}
	    padding-right {}
	    padding-top {
		#-spacing1 ...
	    }
	    text-align {
		set tmap($selector,$property) $value
		lappend tmap($selector) $property
	    }
	    text-decoration {
		switch -- $value {
		    line-through -
		    blink {
			set tmap($selector,$property) $HMstyle_decoration_map($value)
		    }
		    none continue
		    underline -
		    default {
			set tmap($selector,$property) underline
		    }
		}
		lappend tmap($selector) $property
	    }
	    text-indent {}
	    text-transform {
		switch -- $value {
		    uppercase {
			set tmap($selector,$property) toupper
		    }
		    lowercase {
			set tmap($selector,$property) tolower
		    }
		    capitalize {
			# Tricky... not currently supported
			continue
		    }
		    default continue
		}
		lappend tmap($selector) text-transform
	    }
	    vertical-align {}
	    width {}
	    word-spacing {}

	    display {
		set tmap($selector,$property) $value
	    }
	    list-counter -
	    list-style {
		set tmap($selector,$property) $value
		lappend tmap($selector) $property
	    }
	    white-space {
		catch {
		    set tmap($selector,$property) $HMstyle_ws_map($value)
		    lappend tmap($selector) $property
		}
	    }
	}
    }
}	

# HMstyle_default --
#
# A default stylesheet for HTML v2.0/v3.2
#
# This stylesheet is based on Appendix A in the CSS1 Working Draft.

set HMstyle_default {
  BODY { 
    margin: 1em;
    font-family: serif;
    line-height: 1.1;
    background: white;
    color: black; 
  }

  H1, H2, H3, H4, H5, H6, P, 
  DT, DD, BR, CENTER { display: block }
  /* HR is a special case */

  /* There doesn't seem to be a means in CSS1 of specifying nesting block elements.
  Here's a kludge for now: */

  DL, UL, OL, DIR, MENU, DIV, PRE, ADDRESS, BLOCKQUOTE, TABLE { display: block nesting }
  MYUL { display: block nesting; list-style: disc inside }

  B, STRONG, I, U, EM, CITE, VAR, TT, CODE, KBD, SAMP, 
  IMG, SPAN, BLINK { display: inline }

  A { display: inline }

  LI { display: list-item }

  H1, H2, H3, H4 { margin-top: 1em; margin-bottom: 1em }
  H5, H6 { margin-top: 1em }
  H1 { text-align: center }
  H1, H2, H4, H6 { font-weight: bold }
  H3, H5 { font-style: italic }

  H1 { font-size: xx-large }
  H2 { font-size: x-large }
  H3 { font-size: large }

  B, STRONG { font-weight: bolder }  /* relative to the parent */
  I, CITE, EM, VAR, ADDRESS, BLOCKQUOTE { font-style: italic }
  U { text-decoration: underline }
  PRE, TT, CODE, KBD, SAMP { font-family: monospace }
  BLINK { text-decoration: blink }	/* Uggh! */

  CENTER { text-align: center }

  PRE { white-space: pre }

  ADDRESS { margin-left: 3em }
  BLOCKQUOTE { margin-left: 3em; margin-right: 3em }

  UL, DIR { list-style: disc }
  OL { list-style: decimal }

  OL { list-counter: 0 }	/* For internal use */

  MENU { margin: 0 }              /* tight formatting */
  LI { margin-left: 3em }

  DT { margin-bottom: 0 }
  DD { margin-top: 0; margin-left: 3em }

  /* This is handled as a special case */
  /* HR { border-top: solid } */       /* 'border-bottom' could also have been used */

  A:link { color: blue }          /* unvisited link */
  A:visited { color: olive }        /* visited links */
  A:active { color: lime; border: 2px outset lime }        /* active links */

  /* setting the anchor border around IMG elements
     requires contextual selectors */

  A:link IMG { border: 2px solid blue }
  A:visited IMG { border: 2px solid red }
  A:active IMG { border: 2px solid lime }
}

# HMcvtLength --
#
# Converts a CSS1 unit of measure to pixels, returning whether
# the measure is absolute or relative.
#
# Arguments:
#	unit	CSS1 unit measure
#	len	pass-by-reference variable for pixel length
#
# Return Value:
#	"absolute" or "relative" appropriately.
#	Returns empty string if length not recognised.

proc HMcvtLength {value len} {
    upvar $len length
    if {[regexp {([^0-9])%$} $value all length]} {
	return relative
    } elseif {[regexp {(\+|-)?([0-9\.]+)(.*)} $value all sign length units]} {
	if {$sign == "-"} {set length "$sign$length"}
	# Convert to pixels
	switch $units {
	    pt {
		set length [expr $length * 10]
		return absolute
	    }
	    pc {
		set length [expr $length * 12 * 10]
		return absolute
	    }
	    {} -
	    px {return absolute}
	    in {
		set length [expr $length * 10 * 72]
		return absolute
	    }
	    cm {
		set length [expr $length * 10 * 72 / 2.54]
		return absolute
	    }
	    mm {
		set length [expr $length * 10 * 72 / 25.4]
		return absolute
	    }

	    ex -
	    em {
		set length [expr $length * 100]
		return relative
	    }
	}
    } elseif {[string first $value {xx-small x-small small medium large x-large xx-large}]} {
	return absolute
    } elseif {[string first $value {smaller larger}]} {
	return relative
    } else {
	return {}
    }
}

proc HMcvtColour {cmd value} {
    global HMstyle_colour_map
    if [catch {$cmd tag configure colour$value -foreground $value}] {
	if [catch {$cmd tag configure colour$value -foreground $HMstyle_colour_map($value)}] {
	    if [regexp {rgb\(([0-9]+)(%?),([0-9]+)(%?),([0-9]+)(%?)\)} $value all red redpc green greenpc blue bluepc] {
		if [string compare {} $redpc] {
		    set red [expr $red * 2.55]
		}
		if [string compare {} $greenpc] {
		    set green [expr $green * 2.55]
		}
		if [string compare {} $bluepc] {
		    set blue [expr $blue * 2.55]
		}
		set value [format {#%02x%02x%02x} [HMmin [HMmax $red 0] 255] [HMmin [HMmax $green 0] 255] [HMmin [HMmax $blue 0] 255]]
		$cmd tag configure colour$value -foreground $value
	    } else {return {}}
	}
    }
    return colour$value
}

# Utilities.
#
# min & max, which won't allow errors

proc HMmin {a b} {
    catch {expr $a > $b ? $b : $a} err
    return $err
}

proc HMmax {a b} {
    catch {expr $a < $b ? $b : $a} err
    return $err
}

###
### The following code is verbatim from html_library.tcl
###

############################################
# Turn HTML into TCL commands
#   html    A string containing an html document
#   cmd		A command to run for each html tag found
#   start	The name of the dummy html start/stop tags

proc HMparse_html {html {cmd HMtest_parse} {start hmstart}} {
	regsub -all \{ $html {\&ob;} html
	regsub -all \} $html {\&cb;} html
	set w " \t\r\n"	;# white space
	proc HMcl x {return "\[$x\]"}
	set exp <(/?)([HMcl ^$w>]+)[HMcl $w]*([HMcl ^>]*)>
	set sub "\}\n$cmd {\\2} {\\1} {\\3} \{"
	regsub -all $exp $html $sub html
	eval "$cmd {$start} {} {} \{ $html \}"
	eval "$cmd {$start} / {} {}"
}

# Convert multiple white space into a single space

proc HMzap_white {data} {
	regsub -all "\[ \t\r\n\]+" $data " " data
	return $data
}

# find HTML escape characters of the form &xxx;

proc HMmap_esc {text} {
	if {![regexp & $text]} {return $text}
	regsub -all {([][$\\])} $text {\\\1} new
	regsub -all {&#([0-9][0-9]?[0-9]?);?} \
		$new {[format %c [scan \1 %d tmp;set tmp]]} new
	regsub -all {&([a-zA-Z]+);?} $new {[HMdo_map \1]} new
	return [subst $new]
}

# convert an HTML escape sequence into character

proc HMdo_map {text {unknown ?}} {
	global HMesc_map
	set result $unknown
	catch {set result $HMesc_map($text)}
	return $result
}

# table of escape characters (ISO latin-1 esc's are in a different table)

array set HMesc_map {
   lt <   gt >   amp &   quot \"   bsl \\   copy \xa9
   reg \xae   ob \x7b   cb \x7d   nbsp \xa0
}
#############################################################
# ISO Latin-1 escape codes

array set HMesc_map {
	nbsp \xa0 iexcl \xa1 cent \xa2 pound \xa3 curren \xa4
	yen \xa5 brvbar \xa6 sect \xa7 uml \xa8 copy \xa9
	ordf \xaa laquo \xab not \xac shy \xad reg \xae
	hibar \xaf deg \xb0 plusmn \xb1 sup2 \xb2 sup3 \xb3
	acute \xb4 micro \xb5 para \xb6 middot \xb7 cedil \xb8
	sup1 \xb9 ordm \xba raquo \xbb frac14 \xbc frac12 \xbd
	frac34 \xbe iquest \xbf Agrave \xc0 Aacute \xc1 Acirc \xc2
	Atilde \xc3 Auml \xc4 Aring \xc5 AElig \xc6 Ccedil \xc7
	Egrave \xc8 Eacute \xc9 Ecirc \xca Euml \xcb Igrave \xcc
	Iacute \xcd Icirc \xce Iuml \xcf ETH \xd0 Ntilde \xd1
	Ograve \xd2 Oacute \xd3 Ocirc \xd4 Otilde \xd5 Ouml \xd6
	times \xd7 Oslash \xd8 Ugrave \xd9 Uacute \xda Ucirc \xdb
	Uuml \xdc Yacute \xdd THORN \xde szlig \xdf agrave \xe0
	aacute \xe1 acirc \xe2 atilde \xe3 auml \xe4 aring \xe5
	aelig \xe6 ccedil \xe7 egrave \xe8 eacute \xe9 ecirc \xea
	euml \xeb igrave \xec iacute \xed icirc \xee iuml \xef
	eth \xf0 ntilde \xf1 ograve \xf2 oacute \xf3 ocirc \xf4
	otilde \xf5 ouml \xf6 divide \xf7 oslash \xf8 ugrave \xf9
	uacute \xfa ucirc \xfb uuml \xfc yacute \xfd thorn \xfe
	yuml \xff
}

### These are also verbatim from html_library, but must be
### replaced by stylesheet processing

array set HMinsert_map {
	blockquote "\n\n" /blockquote "\n"
	br	"\n"
	dd	"\n" /dd	"\n"
	dl	"\n" /dl	"\n"
	dt	"\n"
	form "\n"	/form "\n"
	h1	"\n\n"	/h1	"\n"
	h2	"\n\n"	/h2	"\n"
	h3	"\n\n"	/h3	"\n"
	h4	"\n"	/h4	"\n"
	h5	"\n"	/h5	"\n"
	h6	"\n"	/h6	"\n"
	li   "\n"
	/dir "\n"
	/ul "\n"
	/ol "\n"
	/menu "\n"
	p	"\n\n"
	pre "\n"	/pre "\n"
}

# generate an X font name
proc HMx_font {family size weight style {adjust_size 0}} {
	catch {incr size $adjust_size}
	return "-*-$family-$weight-$style-normal-*-*-${size}0-*-*-*-*-*-*"
}

# extract a value from parameter list (this needs a re-do)
# returns "1" if the keyword is found, "0" otherwise
#   param:  A parameter list.  It should alredy have been processed to
#           remove any entity references
#   key:    The parameter name
#   val:    The variable to put the value into (use key as default)

proc HMextract_param {param key {val ""}} {

	if {$val == ""} {
		upvar $key result
	} else {
		upvar $val result
	}
    set ws "    \n\r"
 
    # look for name=value combinations.  Either (') or (") are valid delimeters
    if {
      [regsub -nocase [format {.*%s[%s]*=[%s]*"([^"]*).*} $key $ws $ws] $param {\1} value] ||
      [regsub -nocase [format {.*%s[%s]*=[%s]*'([^']*).*} $key $ws $ws] $param {\1} value] ||
      [regsub -nocase [format {.*%s[%s]*=[%s]*([^%s]+).*} $key $ws $ws $ws] $param {\1} value] } {
        set result $value
        return 1
    }

	# now look for valueless names
	# I should strip out name=value pairs, so we don't end up with "name"
	# inside the "value" part of some other key word - some day
	
	set bad \[^a-zA-Z\]+
	if {[regexp -nocase  "$bad$key$bad" -$param-]} {
		return 1
	} else {
		return 0
	}
}
