########################################################################
# fichier poparg.tcl
# 
# Maurice DIAMANTINI diam@ensta.fr
# 
# Version initiale de Frank Mangin mangin@sophia.inria.fr
# 
# xx/12/95 (diam@ensta.fr) : rorganisation
# 06/09/95 (diam@ensta.fr) : modifs pour supporter les tableaux
# 05/05/97 (diam@ensta.fr) : modifs mineurs (prsentation, ...)
#
# A FAIRE : 
#
# - Changer les noms et en faire une partie de package Parse_xxx,
# - Modifier les noms des procdures en consquence,
# - simplifier et acclrer en utilisant la notion d'alias de tcl7.6 :
#      interp alias {} Popargv {} ParseArgl argv
#      interp alias {} Popargs {} ParseArgl args
#   Mais attention : il faut dclarer des proc bidons
#   Popargv et Popargs pour la prise en compte dans le tclIndex !
# - grer la mta-option "--"
# - rajouter une proc "popargvEndSwitch" pour s'assurer qu'il
#   ne reste plus d'option, et pour virer le "--" ventuel
# - envisager d'interdire les switch colles aux valeurs pour 
#   allger les procdures (analyse manuelle toujours possible 
#   pour ces cas l)
# 
# BUG : popargv ne met pas  jour la globale argc.
########################################################################

package provide parsarg 0.1


########################################################################
# Procdures d'extraction de parametres d'une liste. 
#
# Trois procdures sont proposes :
#   - popargv switch valeurs    : analyse la liste globale argv
#   - popargc switch valeurs    : analyse la liste courante args
#   - popargl mylist switch valeurs : analyse la liste mylist
# 
# Description des possibilits de popargv :
# 
# popargv switch valeurs  
#   - Analyse la liste d'argument global argv et en extrait "switch"
#   - retourne 1 si une extraction a t faite, 0 sinon.
#   - affecte ventuellement chaque lment de "valeurs" qui est une 
#     liste dont les lments sont passs par variable.
#   - supprime les lments exploits de la liste analyse.
# 
# Exemples d'utilisation :
# 
# DEBUG recoit 1 ou 0 suivant que $argv contient "-debug" ou non :
#   set DEBUG [popargv {-debug}]   
# 
# VERBOSE recoit 1 su on a pass "-verbose" ou "-debug" :
#   set VERBOSE [expr [popargv {-verbose}] || [popargv {-debug}] ]
# 
# Attention au caractre spciaux (style glob) :
#   if {[popargv {-\?}] || [popargv {-help}]} { PrintUsage ; exit }
# 
# L'option peut tre ou non accole au switch : imprimer -Pmiel
# Mais attention : ceci ncessite d'extraire l'option -POPO avant
# l'option -P !!
#   if {![popargv {-P} Printer]} {set Printer $env(PRINTER)}
# 
# On peut extraire plusieurs arguments  l'aide d'un mme switch.
# popargv retourne une erreur s'il y a moins de 4 valeurs (x1, ...) :
#   if {![popargv {-coords} x1 y1 x2 y2]} { error }
# 
# Pour extraire une valeur simple (sans le tiret) : myscript myfile
#   if {[popargv {} outPath]} {
#     ... 
#   }
# 


########################################################################
# UpvarAlias --
#   level : niveau du contexte parent souhait (comme pour upvar)
#   elems : liste de variables (y compris des lments de tableau)
#
# Rend chaque element de la liste "elems" visible au niveau "level"
# mme si cet lment est un tableau (on ne peut pas faire simplement
# upvar #0 tab(element) tab(element) ).
#
# exemples d'utilisation : 
#   - UpvarAlias 1 $args
#   - UpvarAlias #0 { DEBUG  setup(verboseLevel) }
# La ligne prcdente quivaut  faire :
#   - upvar #0 DEBUG DEBUG
#   - upvar #0 setup setup
# 
proc UpvarAlias {level elems} {

#     # A TEST CE QUI SUIT
#     foreach tabOrVar $elems {
#         regexp {^([^\(]*)\(([^\)]*)\)$} $tabOrVar match tabOrVar elem
#         uplevel 1 [list upvar $level $tabOrVar $tabOrVar]
# 
#     }
    foreach var $elems {
        if [regexp {^([^\(]*)\(([^\)]*)\)$} $var match tab elem] {
            # $var est de la forme "toto(element)" 
            # avec $tab == toto  et  $elem == element 
            uplevel 1 "upvar $level $tab $tab"
        } else {
            # $var est une variable simple
            uplevel 1 "upvar $level $var $var"
        }
    }
} ;#endproc UpvarAlias


########################################################################
# popargv --
#   opt : cl  dtecter (par exemple -d ou -coord)
#   args : plusieurs noms de variable  affetcer
# 
# Extrait les options de la VARIABLE GLOBALE argv 
# Utilise les procdures UpvarAlias et popargl
# 
proc popargv {opt args} {
  global argv
  # Les lments de la liste args doivent tre dfinis  ce niveau pour
  # pouvoir tre accessibles individuellement par la procdure appele.
  # # foreach var $args {upvar 1 $var $var}
  UpvarAlias  1 $args
  
  # Ncessaire car "popargv -debug" est diffrent de "popargv -debug {}" !
  return [eval [list popargl argv $opt] $args]
}

########################################################################
# popargs --
#   opt : cl  dtecter (par exemple -d ou -coord)
#   args : plusieurs noms de variable  affecter
# 
# Extrait les options de la VARIABLE GLOBALE args (et non pas argv) 
# Utilise les procdures UpvarAlias et popargl
# 
# RESTE A VALIDER : JAMAIS TESTEE
# 
proc popargs {opt args} {
  upvar 1 args upargs

  puts stderr "A TESTER avec PLUSIEURS arguments !!!!!!!!!!!!!!"
  exit 1

  # Les lments de la liste args doivent tre dfinis  ce niveau pour
  # pouvoir tre accessibles individuellement par la procdure appele.
  # # foreach var $args {upvar 1 $var $var}
  UpvarAlias  1 $args

  # Ncessaire car "popargs -debug" est diffrent de "popargs -debug {}" !
  return [eval [list popargl upargs $opt] $args]
}

########################################################################
# popargl --
#   arglist : liste d'argument  examiner (et  modifier)
#   opt : cl  dtecter (par exemple -d ou -coord)
#   args : plusieurs noms de variable  affetcer
# 
# Extrait les options de la variable de nom arglist (passe par rfrence) 
# Utilise les procdures UpvarAlias
# 
proc popargl {arglistName opt args} {
  # upvar ?level? upname localname (exemple upvar 1 $param1 p1)
  upvar 1 $arglistName argl
  
  # Chaque lment de args est est associ  une variable du mme nom
  # dans le contexte de la procdure appelante.
  UpvarAlias  1 $args
  
  if {$opt == {}} then {
    #  value alone  #
    foreach var $args {
      if {[set i [lsearch -regexp $argl {^[^-]} ]] < 0} {
        return 0
      } else {
        set $var [lindex $argl $i]
        set argl [lreplace $argl $i $i]
      }
    }
    return 1
  } elseif {[set i [lsearch $argl "$opt*"]] >= 0} then {
      ## found switch ##
      set re [format "%s%s" $opt {([^ ]+)}]
      if {[regexp -- $re [lindex $argl $i] foo value]} {
        # collated switch and value #
        set [lindex $args 0] $value
        set argl [lreplace $argl $i $i]
        return 1
      } else {
        # separate value #
        if {[expr "[llength $argl] - $i - 1"] < [llength $args]} {
          # not enough values for the switch #
          return 0
        } else {
          # everything's ok #
          set argl [lreplace $argl $i $i]
          foreach var $args {
            set $var [lindex $argl $i]
            set argl [lreplace $argl $i $i]
          }
          if {$args==""} then {
          } else {
          }
          return 1
        }
      }
  } else {
      return 0
  }
} ;# end popargl

